@@ -29,10 +29,11 @@
 use twilight_model::{
     application::command::{
         BaseCommandOptionData, ChannelCommandOptionData, ChoiceCommandOptionData, Command,
-        CommandOption, CommandOptionChoice, CommandType, Number, OptionsCommandOptionData,
+        CommandOption, CommandOptionChoice, CommandOptionValue, CommandType, Number,
+        NumberCommandOptionData, OptionsCommandOptionData,
     },
     channel::ChannelType,
-    id::{ApplicationId, CommandId, GuildId},
+    id::{ApplicationId, CommandId, CommandVersionId, GuildId},
 };
 
 /// Builder to create a [`Command`].
@@ -44,7 +45,7 @@ pub struct CommandBuilder(Command);
 impl CommandBuilder {
     /// Create a new default [`Command`] builder.
     #[must_use = "builders have no effect if unused"]
-    pub const fn new(name: String, description: String, kind: CommandType) -> Self {
+    pub fn new(name: String, description: String, kind: CommandType) -> Self {
         Self(Command {
             application_id: None,
             default_permission: None,
@@ -54,6 +55,7 @@ impl CommandBuilder {
             kind,
             name,
             options: Vec::new(),
+            version: CommandVersionId::new(1).expect("non zero"),
         })
     }
 
@@ -204,15 +206,18 @@ impl From<ChannelBuilder> for CommandOption {
 /// Create a integer option with a builder.
 #[derive(Clone, Debug)]
 #[must_use = "should be used in a command builder"]
-pub struct IntegerBuilder(ChoiceCommandOptionData);
+pub struct IntegerBuilder(NumberCommandOptionData);
 
 impl IntegerBuilder {
     /// Create a new default [`IntegerBuilder`].
     #[must_use = "builders have no effect if unused"]
     pub const fn new(name: String, description: String) -> Self {
-        Self(ChoiceCommandOptionData {
+        Self(NumberCommandOptionData {
+            autocomplete: false,
             choices: Vec::new(),
             description,
+            max_value: None,
+            min_value: None,
             name,
             required: false,
         })
@@ -225,10 +230,19 @@ impl IntegerBuilder {
         CommandOption::Integer(self.0)
     }
 
+    /// Set whether this option supports autocomplete.
+    ///
+    /// Defaults to false.
+    pub const fn autocomplete(mut self, autocomplete: bool) -> Self {
+        self.0.autocomplete = autocomplete;
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, i64)` corresponding to the name and value.
-
+    ///
     /// Defaults to no choices.
     pub fn choices(mut self, choices: impl IntoIterator<Item = (String, i64)>) -> Self {
         self.0.choices = choices
@@ -239,6 +253,24 @@ impl IntegerBuilder {
         self
     }
 
+    /// Set the maximum value permitted.
+    ///
+    /// Defaults to no limit.
+    pub const fn max_value(mut self, value: i64) -> Self {
+        self.0.max_value = Some(CommandOptionValue::Integer(value));
+
+        self
+    }
+
+    /// Set the minimum value permitted.
+    ///
+    /// Defaults to no limit.
+    pub const fn min_value(mut self, value: i64) -> Self {
+        self.0.min_value = Some(CommandOptionValue::Integer(value));
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -297,15 +329,18 @@ impl From<MentionableBuilder> for CommandOption {
 /// Create a [`Number`] option with a builder.
 #[derive(Clone, Debug)]
 #[must_use = "should be used in a command builder"]
-pub struct NumberBuilder(ChoiceCommandOptionData);
+pub struct NumberBuilder(NumberCommandOptionData);
 
 impl NumberBuilder {
     /// Create a new default [`NumberBuilder`].
     #[must_use = "builders have no effect if unused"]
     pub const fn new(name: String, description: String) -> Self {
-        Self(ChoiceCommandOptionData {
+        Self(NumberCommandOptionData {
+            autocomplete: false,
             choices: Vec::new(),
             description,
+            max_value: None,
+            min_value: None,
             name,
             required: false,
         })
@@ -318,6 +353,15 @@ impl NumberBuilder {
         CommandOption::Number(self.0)
     }
 
+    /// Set whether this option supports autocomplete.
+    ///
+    /// Defaults to false.
+    pub const fn autocomplete(mut self, autocomplete: bool) -> Self {
+        self.0.autocomplete = autocomplete;
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, Number)` corresponding to the name and
@@ -333,6 +377,24 @@ impl NumberBuilder {
         self
     }
 
+    /// Set the maximum value permitted.
+    ///
+    /// Defaults to no limit.
+    pub const fn max_value(mut self, value: f64) -> Self {
+        self.0.max_value = Some(CommandOptionValue::Number(Number(value)));
+
+        self
+    }
+
+    /// Set the minimum value permitted.
+    ///
+    /// Defaults to no limit.
+    pub const fn min_value(mut self, value: f64) -> Self {
+        self.0.min_value = Some(CommandOptionValue::Number(Number(value)));
+
+        self
+    }
+
     /// Set whether this option is required.
     ///
     /// Defaults to false.
@@ -398,6 +460,7 @@ impl StringBuilder {
     #[must_use = "builders have no effect if unused"]
     pub const fn new(name: String, description: String) -> Self {
         Self(ChoiceCommandOptionData {
+            autocomplete: false,
             choices: Vec::new(),
             description,
             name,
@@ -412,6 +475,15 @@ impl StringBuilder {
         CommandOption::String(self.0)
     }
 
+    /// Set whether this option supports autocomplete.
+    ///
+    /// Defaults to false.
+    pub const fn autocomplete(mut self, autocomplete: bool) -> Self {
+        self.0.autocomplete = autocomplete;
+
+        self
+    }
+
     /// Set the list of choices for an option.
     ///
     /// Accepts tuples of `(String, String)` corresponding to the name and
@@ -645,10 +717,13 @@ mod tests {
                              will be edited"
                                 .into(),
                         ))
-                        .option(NumberBuilder::new(
-                            "position".into(),
-                            "The position of the new role".into(),
-                        )),
+                        .option(
+                            NumberBuilder::new(
+                                "position".into(),
+                                "The position of the new role".into(),
+                            )
+                            .autocomplete(true),
+                        ),
                 ),
         )
         .build();
@@ -661,15 +736,15 @@ mod tests {
             default_permission: None,
             description: String::from("Get or edit permissions for a user or a role"),
             id: None,
-            options: vec![
+            options: Vec::from([
                 CommandOption::SubCommandGroup(OptionsCommandOptionData {
                     description: String::from("Get or edit permissions for a user"),
                     name: String::from("user"),
-                    options: vec![
+                    options: Vec::from([
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Get permissions for a user"),
                             name: String::from("get"),
-                            options: vec![
+                            options: Vec::from([
                                 CommandOption::User(BaseCommandOptionData {
                                     description: String::from("The user to get"),
                                     name: String::from("user"),
@@ -684,12 +759,12 @@ mod tests {
                                     name: String::from("channel"),
                                     required: false,
                                 }),
-                            ],
+                            ]),
                         }),
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Edit permissions for a user"),
                             name: String::from("edit"),
-                            options: vec![
+                            options: Vec::from([
                                 CommandOption::User(BaseCommandOptionData {
                                     description: String::from("The user to edit"),
                                     name: String::from("user"),
@@ -704,18 +779,18 @@ mod tests {
                                     name: String::from("channel"),
                                     required: false,
                                 }),
-                            ],
+                            ]),
                         }),
-                    ],
+                    ]),
                 }),
                 CommandOption::SubCommandGroup(OptionsCommandOptionData {
                     description: String::from("Get or edit permissions for a role"),
                     name: String::from("role"),
-                    options: vec![
+                    options: Vec::from([
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Get permissions for a role"),
                             name: String::from("get"),
-                            options: vec![
+                            options: Vec::from([
                                 CommandOption::Role(BaseCommandOptionData {
                                     description: String::from("The role to get"),
                                     name: String::from("role"),
@@ -730,12 +805,12 @@ mod tests {
                                     name: String::from("channel"),
                                     required: false,
                                 }),
-                            ],
+                            ]),
                         }),
                         CommandOption::SubCommand(OptionsCommandOptionData {
                             description: String::from("Edit permissions for a role"),
                             name: String::from("edit"),
-                            options: vec![
+                            options: Vec::from([
                                 CommandOption::Role(BaseCommandOptionData {
                                     description: String::from("The role to edit"),
                                     name: String::from("role"),
@@ -750,17 +825,21 @@ mod tests {
                                     name: String::from("channel"),
                                     required: false,
                                 }),
-                                CommandOption::Number(ChoiceCommandOptionData {
+                                CommandOption::Number(NumberCommandOptionData {
+                                    autocomplete: true,
                                     choices: Vec::new(),
                                     description: String::from("The position of the new role"),
+                                    max_value: None,
+                                    min_value: None,
                                     name: String::from("position"),
                                     required: false,
                                 }),
-                            ],
+                            ]),
                         }),
-                    ],
+                    ]),
                 }),
-            ],
+            ]),
+            version: CommandVersionId::new(1).expect("non zero"),
         };
 
         assert_eq!(command, command_manual);
